@@ -8,10 +8,11 @@ import psutil
 from firexapp.broker_manager.broker_factory import BrokerFactory
 from socket import gethostname
 from firexapp.common import poll_until_file_not_empty, poll_until_dir_empty, find_procs
-from firexapp.plugins import PLUGGING_ENV_NAME, cdl2list
+from firexapp.plugins import PLUGINS_ENV_NAME, cdl2list
 from firexapp.fileregistry import FileRegistry
 from collections.abc import Iterable
 from firexapp.common import qualify_firex_bin
+from typing import Optional
 
 logger = setup_console_logging(__name__)
 
@@ -75,7 +76,7 @@ class CeleryManager(object):
     @staticmethod
     def get_plugins_env(plugins):
         plugin_files = cdl2list(plugins)
-        return {PLUGGING_ENV_NAME: ",".join(plugin_files)}
+        return {PLUGINS_ENV_NAME: ",".join(plugin_files)}
 
     @staticmethod
     def get_celery_logs_dir(logs_dir):
@@ -219,8 +220,11 @@ class CeleryManager(object):
         pid = self.get_pid_from_file(pid_file)
         self.log('pid %d became active' % pid)
 
-    def start(self, workername, queues=None, wait=True, timeout=15*60, concurrency=None, worker_log_level=None,
-              app=None, cap_concurrency=None, cwd=None, soft_time_limit=None, autoscale: tuple = None):
+    def start(self,
+        workername, queues=None, wait=True, timeout=15*60, concurrency=None, worker_log_level=None,
+        app=None, cap_concurrency=None, cwd=None, soft_time_limit=None, autoscale: Optional[tuple] = None,
+        detach: bool = True,
+    ):
 
         # Override defaults if applicable
         worker_log_level = worker_log_level if worker_log_level else self.worker_log_level
@@ -232,9 +236,17 @@ class CeleryManager(object):
         pid_file = self._get_pid_file(workername)
         self.pid_files[workername] = pid_file
 
-        cmd = '%s --app=%s worker --hostname=%s@%%h --loglevel=%s ' \
-              '--logfile=%s --pidfile=%s --events --without-gossip --without-heartbeat --without-mingle -Ofair' % \
-              (self.celery_bin, app, workername, worker_log_level, log_file, pid_file)
+        cmd = f'{self.celery_bin} ' \
+              f'--app={app} worker ' \
+              f'--hostname={workername}@%h ' \
+              f'--loglevel={worker_log_level} ' \
+              f'--logfile={log_file} ' \
+              f'--pidfile={pid_file} ' \
+              f'--events ' \
+              f'--without-gossip ' \
+              f'--without-heartbeat ' \
+              f'--without-mingle ' \
+              f'-Ofair'
         if queues:
             cmd += ' --queues=%s' % queues
 
@@ -262,7 +274,8 @@ class CeleryManager(object):
             pass
         else:
             cmd += " | ts '[%Y-%m-%d %H:%M:%S]'"
-        cmd += ' &'
+        if detach:
+            cmd += ' &'
 
         self.log('Starting %s on %s...' % (workername, self.hostname))
         self.log(cmd)
@@ -274,7 +287,7 @@ class CeleryManager(object):
             subprocess.check_call(cmd, shell=True, stdout=fp, stderr=subprocess.STDOUT, env=self.env,
                                   cwd=cwd)
 
-        if wait:
+        if detach and wait:
             self.wait_until_active(pid_file=pid_file, timeout=timeout, stdout_file=stdout_file, workername=workername)
 
     @staticmethod
