@@ -12,10 +12,14 @@ from firexapp.engine.celery import app
 from firexkit.resources import get_firex_css_filepath, get_firex_logo_filepath
 from firexkit.firexkit_common import JINJA_ENV
 from celery._state import get_current_task
+from celery.utils import functional
 
 RAW_LEVEL_NAME = 'RAW'
 PRINT_LEVEL_NAME = 'PRINT'
 
+RAW = logging.DEBUG - 5
+PRINT = logging.WARNING + 5
+DEBUG = logging.DEBUG
 
 def add_hostname_to_log_records():
     old_factory = logging.getLogRecordFactory()
@@ -29,19 +33,19 @@ def add_hostname_to_log_records():
 
 
 def log_raw(self, message, *args, **kwargs):
-    self._log(logging.RAW, message, args, **kwargs)
+    self._log(RAW, message, args, **kwargs)
 
 
 def log_print(self, message, *args, **kwargs):
-    self._log(logging.PRINT, message, args, **kwargs)
+    self._log(PRINT, message, args, **kwargs)
 
 
 def add_custom_log_levels():
-    logging.RAW = logging.DEBUG - 5
+    logging.RAW = RAW
     logging.addLevelName(logging.RAW, RAW_LEVEL_NAME)
     logging.Logger.raw = log_raw
 
-    logging.PRINT = logging.WARNING + 5
+    logging.PRINT = PRINT
     logging.addLevelName(logging.PRINT, PRINT_LEVEL_NAME)
     logging.Logger.print = log_print
 
@@ -52,10 +56,10 @@ def add_custom_log_levels():
 
 def add_print_custom_log_level_to_kombu():
     from kombu.log import LOG_LEVELS
-    LOG_LEVELS.setdefault(PRINT_LEVEL_NAME, logging.PRINT)
-    LOG_LEVELS.setdefault(logging.PRINT, PRINT_LEVEL_NAME)
-    LOG_LEVELS.setdefault(RAW_LEVEL_NAME, logging.RAW)
-    LOG_LEVELS.setdefault(logging.RAW, RAW_LEVEL_NAME)
+    LOG_LEVELS.setdefault(PRINT_LEVEL_NAME, PRINT)
+    LOG_LEVELS.setdefault(PRINT, PRINT_LEVEL_NAME)
+    LOG_LEVELS.setdefault(RAW_LEVEL_NAME, RAW)
+    LOG_LEVELS.setdefault(RAW, RAW_LEVEL_NAME)
 
 
 def html_escape(msg):
@@ -131,10 +135,13 @@ class FireXFormatter(celery.utils.log.ColorFormatter):
             return msg
         else:
             original_msg = record.msg
+            original_exc_text = record.exc_text
             if getattr(record, 'html_escape', True):
                 record.msg = html_escape(original_msg)
+                record.exc_text = html_escape(original_exc_text) if original_exc_text else original_exc_text
             msg = super().format(record)
             record.msg = original_msg
+            record.exc_text = original_exc_text
             return msg
 
 
@@ -158,6 +165,7 @@ def configure_task_logger(logger, loglevel, logfile, format, colorize, **_kwargs
     file_handler.addFilter(AddHtmlElementsToLogRecords())
     file_handler.setFormatter(FireXTaskFormatter(format))
 
+
 @after_setup_logger.connect
 def configure_main_logger(logger, loglevel, logfile, format, colorize, **_kwargs):
     # Find the WatchedFileHandler
@@ -181,3 +189,24 @@ def configure_main_logger(logger, loglevel, logfile, format, colorize, **_kwargs
         firex_id=app.conf.uid,
         logs_dir_url=logs_url)
     logger.raw(html_header)
+
+
+class TaskHeaderFilter(logging.Filter):
+    def filter(self, record):
+        if record.funcName == functional.head_from_fun.__name__:
+            return False
+        return True
+
+
+# Filter out useless debug messages printed in functional.head_from_fun()
+functional.logger.addFilter(TaskHeaderFilter())
+
+
+# This is fake, just to ignore attribute access errors.
+class FireXLogger(logging.Logger):
+
+    def print(self, *args, **kwargs):
+        pass
+
+def get_firex_logger(name: str) -> FireXLogger:
+    return celery.utils.log.get_task_logger(name) # noqa
\ No newline at end of file
