@@ -3,15 +3,18 @@ import sys
 import inspect
 import traceback
 from argparse import ArgumentParser, Action
+from types import ModuleType
+from typing import Optional, Union
 
 from celery.signals import worker_init
 from celery.utils.log import get_task_logger
 from firexapp.common import delimit2list
 from firexkit.task import REPLACEMENT_TASK_NAME_POSTFIX
 import importlib.util
+from celery import current_app
 
 logger = get_task_logger(__name__)
-PLUGGING_ENV_NAME = "firex_plugins"
+PLUGINS_ENV_NAME = "firex_plugins"
 
 
 class PluginLoadError(Exception):
@@ -22,32 +25,40 @@ def get_short_name(long_name: str) -> str:
     return long_name.split('.')[-1]
 
 
-def find_plugin_file(file_path):
+def find_plugin_file(file_path: str) -> str:
     # is it a full path?
     if os.path.isabs(file_path):
-        return file_path
-
-    # is it relative to the cwd?
-    in_cwd = os.path.abspath(file_path)
-    if os.path.isfile(in_cwd):
-        return in_cwd
+        plugin_file = file_path
+    else:
+        # Maybe it's relative?
+        plugin_file = os.path.abspath(file_path)
+    if os.path.isfile(plugin_file):
+        return plugin_file
     raise FileNotFoundError(file_path)
 
 
-def cdl2list(plugin_files):
+def convert_plugins_to_list(plugin_files: Union[None, str, list[str]]) -> list[str]:
     if not plugin_files:
         return []
 
-    plugin_files = [file.strip() for file in plugin_files.split(",")]
-    plugin_files = [find_plugin_file(file) for file in plugin_files if file]
+    if not isinstance(plugin_files, list):
+        plugin_files = [
+            file.strip() for file in plugin_files.split(",")
+        ]
+
     return plugin_files
 
 
+def cdl2list(plugin_files: Union[None, str, list[str]]) -> list[str]:
+    plugin_files = convert_plugins_to_list(plugin_files)
+    return [find_plugin_file(file) for file in plugin_files if file]
+
+
 def get_plugin_module_name(plugin_file):
     return os.path.splitext(os.path.basename(plugin_file))[0]
 
 
-def get_plugin_module_names(plugin_files):
+def get_plugin_module_names(plugin_files: Union[None, str, list[str]]) -> list[str]:
     files = cdl2list(plugin_files)
     if not files:
         return []
@@ -71,15 +82,6 @@ def _worker_init_signal(*args, **kwargs):
         exit(-2)
 
 
-def _mark_plugin_module_tasks():
-    from celery import current_app
-    ext_mods = get_plugin_module_names_from_env()
-    for ext_mod in ext_mods:
-        ext_mod_tasks = [t for t in current_app.tasks if t.startswith(ext_mod)]
-        for ext_mod_task in ext_mod_tasks:
-            current_app.tasks[ext_mod_task].from_plugin = True
-
-
 # there is no way of copying the signals without coupling with the internals of celery signals
 # noinspection PyProtectedMember
 def _get_signals_with_connections():
@@ -120,9 +122,8 @@ def create_replacement_task(original, name_postfix, sigs):
                                                        "returns",
                                                        "flame",
                                                        "use_cache",
-                                                       "pending_child_strategy"] if key in dir(original)}
-
-    from celery import current_app
+                                                       "pending_child_strategy",
+                                                       "from_plugin"] if key in dir(original)}
     new_task = current_app.task(name=new_name,
                                 bind=bound,
                                 base=inspect.getmro(original.__class__)[1],
@@ -153,7 +154,6 @@ def create_replacement_task(original, name_postfix, sigs):
 
 def _unregister_duplicate_tasks():
     sigs = _get_signals_with_connections()
-    from celery import current_app
     becomes = identify_duplicate_tasks(current_app.tasks, get_plugin_module_names_from_env())
     for substitutions in becomes:
         prime_overrider = substitutions[-1]  # the last item in the list is the last override
@@ -183,71 +183,129 @@ def identify_duplicate_tasks(all_tasks, priority_modules: list) -> [[]]:
             return priority_modules.index(os.path.splitext(micro_name)[0])
         except ValueError:
             return -1
-    overrides = [sorted(long_names, key=priority_index) for long_names in unique_names.values() if len(long_names) > 1]
+
+    overrides = [
+        sorted(long_names, key=priority_index)
+        for long_names in unique_names.values()
+        if len(long_names) > 1
+    ]
     return overrides
 
 
-def import_plugin_file(plugin_file):
+def _should_import(module_name: str, plugin_file: str, replace: bool) -> tuple[bool, Optional[ModuleType]]:
+    already_loaded = None
+    if module_name in sys.modules:
+        # a module with this name is already loaded. See if we should replace it.
+        existing_mod = sys.modules[module_name]
+        module_source = existing_mod.__file__
+        if module_source != plugin_file:
+            if not replace:
+                logger.error(f'Plugin module {module_name!r} was NOT imported from {plugin_file!r}. '
+                             f'A module with the same name was already imported from {module_source!r}')
+                should_import = False
+            else:
+                logger.warning(
+                    f'Plugin module {module_name!r} already loaded from {module_source!r}. '
+                    f'Will replace with module from {plugin_file!r}'
+                )
+                should_import = True
+        else:
+            logger.warning(f'Plugin module {module_name!r} was already imported from {module_source!r}.')
+            should_import = False
+            already_loaded = existing_mod
+    else:
+        # new module name, always import.
+        should_import = True
 
-    def _import_plugin(module_name, plugin_file):
-        spec = importlib.util.spec_from_file_location(module_name, plugin_file)
-        module = importlib.util.module_from_spec(spec)
-        module_directory = os.path.dirname(os.path.abspath(plugin_file))
-        if module_directory not in sys.path:
-            sys.path.append(module_directory)
-        mod = sys.modules[module_name] = module
-        try:
-            spec.loader.exec_module(module)
-        except BaseException:
-            raise PluginLoadError(f'Fatal Error loading plugin {plugin_file!r}')
-        print(f"Plugins module {module_name!r} imported from {plugin_file!r}")
-        return mod
+    return should_import, already_loaded
+
+
+def _import_plugin(module_name, plugin_file):
+    spec = importlib.util.spec_from_file_location(module_name, plugin_file)
+    module = importlib.util.module_from_spec(spec)
+    module_directory = os.path.dirname(os.path.realpath(plugin_file))
+    if module_directory not in sys.path:
+        sys.path.append(module_directory)
+    mod = sys.modules[module_name] = module
+    try:
+        spec.loader.exec_module(module)
+    except BaseException:
+        logger.exception(f'Failed to load {plugin_file}')
+        raise PluginLoadError(f'Fatal Error loading plugin {plugin_file!r}')
+    return mod
+
+
+def import_plugin_file(plugin_file, replace=False) -> Optional[ModuleType]:
 
     plugin_file = find_plugin_file(plugin_file)
     module_name = get_plugin_module_name(plugin_file)
-
-    if module_name in sys.modules:
-        module_source = sys.modules[module_name].__file__
-        if module_source != plugin_file:
-            logger.error(f'Plugin module {module_name!r} was NOT imported from {plugin_file!r}. '
-                         f'A module with the same name was already imported from {module_source!r}')
-        else:
-            logger.warning(f'Plugin module {module_name!r} was already imported from {module_source!r}.')
-    else:
+    should_import, existing_module = _should_import(module_name, plugin_file, replace)
+    if should_import:
         return _import_plugin(module_name, plugin_file)
 
+    return existing_module
 
-def import_plugin_files(plugin_files):
+
+def import_plugin_files(plugin_files) -> set[str]:
     plugin_files = cdl2list(plugin_files)
     if not plugin_files:
-        return
+        return set()
+
+    imported_module_names = []
+    new_tasks = set()
+    previous_tasks = set(current_app.tasks)
     for plugin_file in plugin_files:
-        import_plugin_file(plugin_file)
+        mod = import_plugin_file(plugin_file)
+        current_tasks = set(current_app.tasks)
+        new_tasks_from_this_import = current_tasks - previous_tasks
+        new_tasks.update(new_tasks_from_this_import)
+        new_tasks_modules_from_this_import = {t.rsplit('.', 1)[0] for t in new_tasks_from_this_import}
+        imported_module_names += list(new_tasks_modules_from_this_import)
+        previous_tasks = current_tasks
+        plugin_modules_info = f'{list(new_tasks_modules_from_this_import)}' + ' ' \
+            if new_tasks_modules_from_this_import \
+            else ""
+        print(f'{len(new_tasks_from_this_import)} new service{"s" if len(new_tasks_from_this_import)>1 else ""} '
+              f'imported from plugin modules {plugin_modules_info}'
+              f'found in {mod.__file__ if mod else plugin_file}')
+
+    if len(plugin_files)>1:
+        if imported_module_names:
+            print(f'--> {len(new_tasks)} total new service{"s" if len(new_tasks)>1 else ""} imported '
+                  f'from {len(imported_module_names)} plugin module{"s" if len(imported_module_names)>1 else ""} '
+                  f'{imported_module_names}')
+        elif not imported_module_names:
+            print(f'No new services imported from {plugin_files}!')
+
+    return new_tasks
 
 
 def set_plugins_env(plugin_files):
     plugin_files = cdl2list(plugin_files)
-    os.environ[PLUGGING_ENV_NAME] = ",".join(plugin_files)
+    os.environ[PLUGINS_ENV_NAME] = ",".join(plugin_files)
 
 
-def get_active_plugins():
-    return os.environ.get(PLUGGING_ENV_NAME, "")
+def get_active_plugins() -> str:
+    return os.environ.get(PLUGINS_ENV_NAME, "")
 
 
 def load_plugin_modules(plugin_files):
     set_plugins_env(plugin_files)
-    import_plugin_files(plugin_files)
+    new_tasks_imported = import_plugin_files(plugin_files)
+    # Mark the newly imported tasks with "from_plugin"
+    for t in new_tasks_imported:
+        current_app.tasks[t].from_plugin = True
     if plugin_files:
         _unregister_duplicate_tasks()
-        _mark_plugin_module_tasks()
 
 
 def load_plugin_modules_from_env():
     plugin_files = get_active_plugins()
-    load_plugin_modules(plugin_files)
+    if plugin_files:
+        load_plugin_modules(plugin_files)
 
 
-def merge_plugins(*plugin_lists) -> []:
+def merge_plugins(*plugin_lists) -> list[str]:
     """Merge  comma delimited lists of plugins into a single list. Right-handed most significant plugin"""
     combined_list = []
     for plugin_list in plugin_lists:
