@@ -6,6 +6,7 @@ import time
 from psutil import Process, TimeoutExpired
 from collections import namedtuple
 from typing import Optional
+import tempfile
 
 from celery import Celery
 import redis.exceptions
@@ -14,8 +15,8 @@ import kombu.exceptions
 from firexapp.celery_manager import CeleryManager
 from firexapp.submit.uid import Uid
 from firexapp.broker_manager.broker_factory import BrokerFactory, REDIS_BIN_ENV
-from firexkit.inspect import get_active, get_revoked, get_active_queues
 from firexapp.common import qualify_firex_bin, select_env_vars
+from firexkit.inspect import get_active, get_revoked, ping
 
 logger = logging.getLogger(__name__)
 
@@ -24,22 +25,27 @@ DEFAULT_CELERY_SHUTDOWN_TIMEOUT = 5 * 60
 MaybeCeleryActiveTasks = namedtuple('MaybeCeleryActiveTasks', ['celery_read_success', 'active_tasks'])
 
 
-def _launch_shutdown_subprocess(shutdown_cmd: list[str]) -> int:
+def _launch_shutdown_subprocess(shutdown_cmd: list[str], logs_dir: str) -> int:
     shutdown_subprocess_env = select_env_vars([REDIS_BIN_ENV, 'PATH'])
+    shutdown_cwd = logs_dir if os.path.isdir(logs_dir) else tempfile.gettempdir()
     try:
-        import detach
+        import detach # noqa
     except ModuleNotFoundError:
         # don't break old installs that don't have detach
+
         return subprocess.Popen(
             shutdown_cmd,
             close_fds=True,
             env=shutdown_subprocess_env,
             preexec_fn=os.setpgrp,
+            # Shutdown doesn't care about cwd, but Celery can crash with bad cwd :/
+            cwd=shutdown_cwd,
         ).pid
     else:
         return detach.call(
             shutdown_cmd,
             env=shutdown_subprocess_env,
+            cwd=shutdown_cwd,
         )
 
 
@@ -50,7 +56,7 @@ def launch_background_shutdown(logs_dir, reason, celery_shutdown_timeout=DEFAULT
                         "--celery_shutdown_timeout", str(celery_shutdown_timeout)]
         if reason:
             shutdown_cmd += ['--reason', reason]
-        pid = _launch_shutdown_subprocess(shutdown_cmd)
+        pid = _launch_shutdown_subprocess(shutdown_cmd, logs_dir)
     except Exception:
         logger.exception("SHUTDOWN PROCESS FAILED TO LAUNCH -- REDIS WILL LEAK.")
         raise
@@ -91,8 +97,8 @@ def _inspect_broker_safe(inspect_fn, broker, celery_app, **kwargs):
         return None
     try:
         return inspect_fn(inspect_retry_timeout=4, celery_app=celery_app, **kwargs)
-    except (redis.exceptions.ConnectionError, kombu.exceptions.DecodeError) as e:
-        logger.warning(e)
+    except (redis.exceptions.ConnectionError, kombu.exceptions.DecodeError) as ex:
+        logger.warning(ex)
         return None
 
 
@@ -103,16 +109,13 @@ def get_active_broker_safe(broker, celery_app):
                                 method_args=(True,))
 
 
-def is_celery_responsive(broker, celery_app):
-    # use get_active_queues instead of get_active to check Celery responsiveness
-    # because get_active can fail to deserialize its response (i.e. raise DecodeError)
-    return _inspect_broker_safe(get_active_queues, broker, celery_app)
-
-
-def get_revoked_broker_safe(broker, celery_app):
-    if not broker.is_alive():
-        return None
-    return get_revoked(inspect_retry_timeout=4, celery_app=celery_app)
+def is_celery_responsive(broker, celery_app) -> bool:
+    # use ping instead of get_active to check Celery responsiveness
+    # because get_active can fail to deserialize its response (i.e. raise DecodeError).
+    # Need some celery call that uses broadcast, since we'll call shutdown that uses
+    # broadcast.
+    r =  _inspect_broker_safe(ping, broker, celery_app)
+    return bool(r)
 
 
 def _tasks_from_active(active, task_predicate) -> MaybeCeleryActiveTasks:
@@ -161,8 +164,10 @@ def revoke_active_tasks(broker, celery_app,  max_revoke_retries=5, task_predicat
     elif len(maybe_active_tasks.active_tasks) == 0:
         logger.info("Confirmed no active tasks after revoke.")
     elif revoke_retries >= max_revoke_retries:
-        logger.warning("Exceeded max revoke retry attempts, %s active tasks may not be revoked."
-                       % len(maybe_active_tasks.active_tasks))
+        t_count = len(maybe_active_tasks.active_tasks)
+        logger.warning(
+            f"Exceeded max revoke retry attempts, {t_count} active tasks may not be revoked."
+        )
 
 
 def init():
@@ -186,7 +191,7 @@ def init():
     return logs_dir, args.reason, args.celery_shutdown_timeout
 
 
-def shutdown_run(logs_dir, celery_shutdown_timeout, reason='No reason provided'):
+def _shutdown_run(logs_dir, celery_shutdown_timeout, reason='No reason provided'):
     logger.info(f"Shutting down due to reason: {reason}")
     logger.info(f"Shutting down with logs: {logs_dir}.")
     broker = BrokerFactory.broker_manager_from_logs_dir(logs_dir)
@@ -195,13 +200,18 @@ def shutdown_run(logs_dir, celery_shutdown_timeout, reason='No reason provided')
     celery_manager = CeleryManager(logs_dir=logs_dir, broker=broker)
     celery_app = Celery(broker=broker.broker_url,
                         accept_content=['pickle', 'json'])
-
     try:
         if is_celery_responsive(broker, celery_app):
             revoke_active_tasks(broker, celery_app)
 
-            logger.info("Found active Celery; sending Celery shutdown.")
-            celery_app.control.shutdown()
+            if is_celery_responsive(broker, celery_app):
+                # double check celery responsiveness since control.shutdown()
+                # can hang when broker is unresponsive, and broker might go
+                # down between is_celery_responsive
+                logger.info("Found active Celery; sending Celery shutdown.")
+                celery_app.control.shutdown()
+            else:
+                logger.info("Celery appears unresponsive")
 
             celery_shutdown_success = celery_manager.wait_for_shutdown(celery_shutdown_timeout)
             if not celery_shutdown_success:
@@ -228,7 +238,7 @@ def shutdown_run(logs_dir, celery_shutdown_timeout, reason='No reason provided')
 def main() -> None:
     logs_dir, reason, celery_shutdown_timeout = init()
     try:
-        shutdown_run(logs_dir, celery_shutdown_timeout, reason)
+        _shutdown_run(logs_dir, celery_shutdown_timeout, reason)
     except BaseException:
         logger.exception("Shutdown failed.")
         raise
