@@ -1,4 +1,5 @@
 import multiprocessing
+import pathlib
 import re
 import sys
 import json
@@ -8,6 +9,8 @@ import argparse
 import time
 import traceback
 from getpass import getuser
+import socket
+from typing import Optional
 
 from celery.signals import worker_ready
 from shutil import copyfile
@@ -28,14 +31,16 @@ from firexapp.submit.arguments import InputConverter, ChainArgException, get_cha
 from firexapp.submit.tracking_service import get_tracking_services, get_service_name
 from firexapp.plugins import plugin_support_parser
 from firexapp.submit.console import setup_console_logging
-from firexapp.application import import_microservices, get_app_tasks, get_app_task, JSON_ARGS_PATH_ARG_NAME
+from firexapp.application import (
+    import_microservices, get_app_tasks, get_app_task, JSON_ARGS_PATH_ARG_NAME
+)
 from firexapp.engine.celery import app
 from firexapp.broker_manager.broker_factory import BrokerFactory
 from firexapp.submit.shutdown import launch_background_shutdown, DEFAULT_CELERY_SHUTDOWN_TIMEOUT
 from firexapp.submit.install_configs import load_new_install_configs, FireXInstallConfigs, INSTALL_CONFIGS_ENV_NAME
 from firexapp.submit.arguments import whitelist_arguments
 from firexapp.common import dict2str, silent_mkdir, create_link
-from firexapp.reporters.json_reporter import FireXJsonReportGenerator
+from firexapp.reporters.json_reporter import FireXJsonReportGenerator, FireXRunData
 
 add_hostname_to_log_records()
 logger = setup_console_logging(__name__)
@@ -47,6 +52,9 @@ FileRegistry().register_file(SUBMISSION_FILE_REGISTRY_KEY, os.path.join(Uid.debu
 ENVIRON_FILE_REGISTRY_KEY = 'env'
 FileRegistry().register_file(ENVIRON_FILE_REGISTRY_KEY, os.path.join(Uid.debug_dirname, 'environ.json'))
 
+RUN_COMPLETE_REGISTRY_KEY = 'RUN_COMPLETE_REGISTRY_KEY'
+FileRegistry().register_file(RUN_COMPLETE_REGISTRY_KEY, os.path.join(Uid.debug_dirname, 'RUN_COMPLETE'))
+
 RUN_SOFT_TIME_LIMIT_KEY = 'run_soft_time_limit'
 
 class JsonFileAction(argparse.Action):
@@ -82,20 +90,30 @@ class AdjustCeleryConcurrency(argparse.Action):
         setattr(namespace, self.dest, concurrency)
 
 
-def safe_create_completed_run_json(chain_result, run_revoked, chain_args):
-    try:
-        FireXJsonReportGenerator.create_completed_run_json(root_id=chain_result,
-                                                           run_revoked=run_revoked,
-                                                           **chain_args)
-    except Exception as e:
-        logger.error(f'Failed to generate completion run JSON: {e}')
+def _safe_create_completed_run_json(uid, chain_result, run_revoked, chain_args, shutdown_reason):
+    if uid:
+        chain_args = dict(chain_args or {})
+        for k in ['uid', 'root_id', 'run_revoked', 'shutdown_reason']:
+            chain_args.pop(k, None) # gotta love **chain_args
+        try:
+            FireXJsonReportGenerator.create_completed_run_json(
+                uid=uid,
+                root_id=chain_result,
+                run_revoked=run_revoked,
+                shutdown_reason=shutdown_reason,
+                **chain_args)
+        except Exception as e:
+            logger.exception(f'Failed to generate completion run JSON: {e}')
+    else:
+        logger.warning("No uid; run.json will not be updated.")
 
 
-def safe_create_initial_run_json(**kwargs):
+def safe_create_initial_run_json(**kwargs) -> Optional[FireXRunData]:
     try:
-        FireXJsonReportGenerator.create_initial_run_json(**kwargs)
+        return FireXJsonReportGenerator.create_initial_run_json(**kwargs)
     except Exception as e:
-        logger.error(f'Failed to generate initial run JSON: {e}')
+        logger.exception(f'Failed to generate initial run JSON: {e}')
+        return None
 
 
 class SubmitBaseApp:
@@ -134,6 +152,7 @@ class SubmitBaseApp:
             submission_log_handler.setFormatter(logging.Formatter(fmt=self.SUBMISSION_LOGGING_FORMATTER,
                                                                   datefmt="%Y-%m-%d %H:%M:%S"))
             submission_log_handler.setLevel(logging.NOTSET)
+            submission_log_handler.set_name('firex_submission_handler')
             root_logger = logging.getLogger()
             root_logger.addHandler(submission_log_handler)
         self.log_preamble()
@@ -183,8 +202,12 @@ class SubmitBaseApp:
                                    default=DEFAULT_CELERY_SHUTDOWN_TIMEOUT, type=int)
         submit_parser.add_argument(JSON_ARGS_PATH_ARG_NAME,
                                    help='Specify submit arguments via a JSON file containing a list of argument names and values.')
-
-
+        submit_parser.add_argument('--save_redis_db',
+                                   nargs='?', const=True, default=False, action=OptionalBoolean,
+                                   help='Save the redis db to file', )
+        submit_parser.add_argument('--redis_server_extra_opts',
+                                   default=os.environ.get('redis_server_extra_opts'),
+                                   help='Extra options for redis-server')
         submit_parser.set_defaults(func=self.run_submit)
 
         for service in get_tracking_services():
@@ -193,8 +216,17 @@ class SubmitBaseApp:
         return submit_parser
 
     def convert_chain_args(self, chain_args) -> dict:
-        with self.graceful_exit_on_failure("The arguments you provided have errors."):
-            return InputConverter.convert(**chain_args)
+        with self.graceful_exit_on_failure("The arguments you provided have errors:"):
+            converted_args = InputConverter.convert(**chain_args)
+            chain_excluded_infra_args = [
+                # 'json_file', # should be exlcuded too since just top-level infra (start/complete)
+                # should be reasoning with 'json_file', but it's embedded in chain_args. Ideally
+                # chain args would be the args received by the chain in --chain <chain>, but
+                # infra args are also in there.
+                'json_args_path']
+            for a in chain_excluded_infra_args:
+                converted_args.pop(a, None)
+            return converted_args
 
     def run_submit(self, args, others):
         self.is_sync = args.sync
@@ -288,6 +320,11 @@ class SubmitBaseApp:
 
         return args, others
 
+    def create_chain_sentinel_files(self, chain: str):
+        top_level_chains = chain.split(',')
+        for c in top_level_chains:
+            pathlib.Path(self.uid.debug_dir,    f'{c.lower()}.chain').touch()
+
     def submit(self, args_from_first_pass: argparse.Namespace, other_args_from_first_pass: list):
         uid = Uid()
         self.uid = uid
@@ -309,14 +346,15 @@ class SubmitBaseApp:
             uid.add_viewers(logs_url=self.install_configs.get_logs_root_url())
             logger.info(f'Logs URL: {uid.logs_url}')
 
-        # Create an env file for debugging
-        with open(FileRegistry().get_file(ENVIRON_FILE_REGISTRY_KEY, self.uid.logs_dir), 'w') as f:
-            json.dump(dict(os.environ), fp=f, skipkeys=True, sort_keys=True, indent=4)
+        self.dump_environ()
 
         chain_args = self.convert_chain_args(chain_args)
 
         chain_args = self.start_engine(args=args, chain_args=chain_args, uid=uid)
 
+        # Write .chain sentinel files
+        self.create_chain_sentinel_files(chain_args['chain'])
+
         # Execute chain
         try:
             root_task_name = app.conf.get("root_task")
@@ -347,6 +385,17 @@ class SubmitBaseApp:
             self.self_destruct(chain_details=(root_task_result_promise, chain_args),
                                reason="Sync run: completed successfully")
 
+    def dump_environ(self):
+        # Mask  any password-related env vars before dumping them in the environ.json
+        copy_of_os_environ = os.environ.copy()
+        for k in copy_of_os_environ:
+            if any(e in k.lower() for e in ['passwd', 'password', 'secret']):
+                copy_of_os_environ[k] = '********'
+
+        # Create an env file for debugging
+        with open(FileRegistry().get_file(ENVIRON_FILE_REGISTRY_KEY, self.uid.logs_dir), 'w') as f:
+            json.dump(copy_of_os_environ, fp=f, skipkeys=True, sort_keys=True, indent=4)
+
     def check_for_failures(self, root_task_result_promise, unsuccessful_services):
         if unsuccessful_services:
             msg, rc = format_unsuccessful_services(unsuccessful_services)
@@ -376,7 +425,9 @@ class SubmitBaseApp:
 
             # IMPORT ALL THE MICROSERVICES
             # ONLY AFTER BROKER HAD STARTED
-            all_tasks = import_microservices(chain_args.get("plugins", args.plugins))
+            all_tasks, plugin_path_mapping = import_microservices(chain_args.get("plugins", args.plugins))
+            if plugin_path_mapping:
+                chain_args['plugin_path_mapping'] = plugin_path_mapping
         except FileNotFoundError as e:
             logger.error("\nError: FireX run failed. File %s is not found." % e)
             self.main_error_exit_handler(reason=str(e))
@@ -389,7 +440,7 @@ class SubmitBaseApp:
 
         # locate task objects
         try:
-            app_tasks = get_app_tasks(args.chain)
+            app_tasks = get_app_tasks(chain_args['chain'])
         except NotRegistered as e:
             reason = "Could not find task %s" % str(e)
             logger.error(reason)
@@ -418,12 +469,12 @@ class SubmitBaseApp:
             self.main_error_exit_handler(reason=str(e))
             sys.exit(-1)
 
-        with self.graceful_exit_on_failure("Failed to start tracking service"):
+        with self.graceful_exit_on_failure("Failed to start tracking service:"):
             # Start any tracking services to monitor, track, and present the state of the run
             chain_args.update(self.start_tracking_services(args, **chain_args))
 
         # Start Celery
-        with self.graceful_exit_on_failure("Unable to start Celery."):
+        with self.graceful_exit_on_failure("Unable to start Celery:"):
             self.start_celery(args, chain_args.get("plugins", args.plugins))
         return chain_args
 
@@ -458,14 +509,14 @@ class SubmitBaseApp:
         chain_args['argv'] = sys.argv
         chain_args['json_file'] = args.json_file
 
-        whitelist_arguments(['submitter', 'submission_dir', 'argv'])
+        whitelist_arguments(['submitter', 'submission_dir', 'argv', 'plugin_path_mapping'])
 
         return chain_args
 
     def start_broker(self, args):
         from firexapp.broker_manager.broker_factory import BrokerFactory
         self.broker = BrokerFactory.create_new_broker_manager(logs_dir=self.uid.logs_dir)
-        self.broker.start()
+        self.broker.start(save_db=args.save_redis_db, redis_server_extra_opts=args.redis_server_extra_opts)
 
     def start_tracking_services(self, args, **chain_args) -> {}:
         assert self.enabled_tracking_services is None, "Cannot start tracking services twice."
@@ -539,7 +590,8 @@ class SubmitBaseApp:
     def wait_tracking_services_task_ready(self, timeout=5)->None:
         self.wait_tracking_services_pred(lambda s: s.ready_for_tasks(celery_app=app), 'ready for tasks', timeout)
 
-    def wait_tracking_services_release_console_ready(self, timeout=5)->None:
+    # increase timeout until Flame's 'recalc' at during shutdown can be removed.
+    def wait_tracking_services_release_console_ready(self, timeout=45)->None:
         self.wait_tracking_services_pred(lambda s: s.ready_release_console(), 'ready to release console', timeout)
 
     def main_error_exit_handler(self, chain_details=None, reason=None, run_revoked=False):
@@ -547,34 +599,58 @@ class SubmitBaseApp:
         if reason:
             mssg += '\n' + str(reason)
         logger.error(mssg)
-        if self.broker:
-            self.self_destruct(chain_details=chain_details, reason=reason, run_revoked=run_revoked)
+        self.self_destruct(chain_details=chain_details, reason=reason, run_revoked=run_revoked)
         if self.uid:
             self.copy_submission_log()
 
     def self_destruct(self, chain_details=None, reason=None, run_revoked=False):
-        if chain_details:
+        if not chain_details:
+            chain_result = chain_args = None
+        else:
             chain_result, chain_args = chain_details
-            safe_create_completed_run_json(chain_result, run_revoked, chain_args)
-            try:
-                logger.debug("Generating reports")
-                from firexapp.submit.reporting import ReportersRegistry
-                ReportersRegistry.post_run_report(results=chain_result,
-                                                  kwargs=chain_args)
-                logger.debug('Reports successfully generated')
-            except Exception:
-                # Under no circumstances should report generation prevent celery and broker cleanup
-                logger.error('Error in generating reports', exc_info=True)
-            finally:
-                # AsyncResult objects access self.backend when garbage collected. Since we're about to initiate a
-                # process to stop the backend, prevent all AsyncResult objects from accessing self.backend.
-                if is_async_result_monkey_patched_to_track():
-                    disable_all_async_results()
-                elif chain_result:
-                    disable_async_result(chain_result)
 
-        logger.debug("Running FireX self destruct")
-        launch_background_shutdown(self.uid.logs_dir, reason, getattr(self.submit_args, 'celery_shutdown_timeout', DEFAULT_CELERY_SHUTDOWN_TIMEOUT))
+        _safe_create_completed_run_json(
+            self.uid, chain_result, run_revoked, chain_args, reason
+        )
+        if self.broker:
+            if chain_result:
+                try:
+                    logger.debug("Generating reports")
+                    from firexapp.submit.reporting import ReportersRegistry
+                    ReportersRegistry.post_run_report(
+                        results=chain_result,
+                        kwargs=chain_args)
+                    logger.debug('Reports successfully generated')
+                except Exception:
+                    # Under no circumstances should report generation prevent celery and broker cleanup
+                    logger.error('Error in generating reports', exc_info=True)
+                finally:
+                    # AsyncResult objects access self.backend when garbage collected. Since we're about to initiate a
+                    # process to stop the backend, prevent all AsyncResult objects from accessing self.backend.
+                    if is_async_result_monkey_patched_to_track():
+                        disable_all_async_results()
+                    elif chain_result:
+                        disable_async_result(chain_result)
+
+            logger.debug("Running FireX self destruct")
+            launch_background_shutdown(
+                self.uid.logs_dir,
+                reason,
+                getattr(
+                    self.submit_args,
+                    'celery_shutdown_timeout',
+                    DEFAULT_CELERY_SHUTDOWN_TIMEOUT))
+
+        self.write_run_complete_file(self.uid.logs_dir)
+
+    @staticmethod
+    def write_run_complete_file(log_path: str):
+        completion_file = FileRegistry().get_file(RUN_COMPLETE_REGISTRY_KEY, log_path)
+        logger.debug(f'Writing {completion_file}')
+        try:
+            pathlib.Path(completion_file).touch()
+        except FileNotFoundError:
+            logger.debug(f"Couldn't write {completion_file}", exc_info=True)
 
     @classmethod
     def validate_argument_applicability(cls, chain_args, args, all_tasks):
@@ -603,17 +679,18 @@ class SubmitBaseApp:
         try:
             yield
         except Exception as e:
-            logger.debug(traceback.format_exc())
+            logger.debug(e, exc_info=True)
             self.error_banner(e, banner_title=failure_caption)
-            self.main_error_exit_handler(reason=failure_caption)
+            self.main_error_exit_handler(reason=f'{failure_caption} {e}')
             sys.exit(-1)
 
 
-def get_firex_id_from_output(cmd_output: str)->str:
+def get_firex_id_from_output(cmd_output: str) -> Optional[str]:
     for line in cmd_output.splitlines():
         match = re.match('.*FireX ID: (.*)', line)
         if match:
             return match.group(1)
+    return None
 
 
 def get_log_dir_from_output(cmd_output: str)->str:
