@@ -21,6 +21,13 @@ class FlowTestConfiguration(object):
     def name(self):
         return self.__class__.__name__
 
+    @property
+    def completed_run(self) -> FireXRunData:
+        assert self.run_data
+        if self.run_data.completed:
+            return self.run_data
+        return self.run_data.reload()
+
     @abc.abstractmethod
     def initial_firex_options(self)->list:
         pass
@@ -36,6 +43,9 @@ class FlowTestConfiguration(object):
     def cleanup(self):
         pass
 
+    @staticmethod
+    def get_extra_run_env():
+        return {}
 
 class InterceptFlowTestConfiguration(FlowTestConfiguration):
     __metaclass__ = abc.ABCMeta
@@ -44,7 +54,7 @@ class InterceptFlowTestConfiguration(FlowTestConfiguration):
         assert 0 == ret_value, "Expected return code of 0"
 
     def assert_expected_firex_output(self, cmd_output, cmd_err):
-        return cmd_err is None, "Intercept tests should not have errors"
+        assert not cmd_err, "Intercept tests should not have errors:\n{cmd_err}"
 
     @abc.abstractmethod
     def intercept_service(self)->str:
@@ -86,6 +96,9 @@ def discover_tests(tests, config_filter="") -> list:
         filters = [config_filter.strip() for config_filter in config_filter.split(",")]
         configs = [config for config in configs if config.__class__.__name__ in filters]
 
+    if not configs:
+        raise Exception(f'No test configs found in {tests}')
+
     [print("Skipping " + config.__class__.__name__, file=sys.stderr) for config in configs if hasattr(config,
                                                                                                       "skip_test")]
     configs = [config for config in configs if not hasattr(config, "skip_test")]
