@@ -7,8 +7,7 @@ from datetime import datetime
 from typing import Optional, List
 import tempfile
 
-from firexapp.common import wait_until
-from firexapp.reporters.json_reporter import load_completion_report
+from firexapp.reporters.json_reporter import FireXRunData
 from firexkit.resources import get_cloud_ci_install_config_path
 from firexapp.submit.submit import get_firex_id_from_output, get_log_dir_from_output
 from firexapp.submit.tracking_service import has_flame
@@ -19,13 +18,10 @@ from firexapp.testing.config_base import InterceptFlowTestConfiguration, FlowTes
 class ConfigInterpreter:
     execution_directory = None
 
-    tmp_json_file: Optional[str]
-
     def __init__(self):
         self.profile = False
         self.coverage = False
         self.is_public = False
-        self.tmp_json_file = None
 
     @staticmethod
     def is_submit_command(test_config: FlowTestConfiguration):
@@ -123,9 +119,6 @@ def {0}(**kwargs):
                 # TODO: should merge test-specific install_configs with ci-viewer configs,
                 #  since we usually want the ci URLs, even with a test's install_config specifies other stuff.
                 cmd += ['--install_configs', get_cloud_ci_install_config_path()]
-            if '--json_file' not in cmd:
-                self.tmp_json_file = tempfile.mktemp(prefix='firex_json_file_')
-                cmd += ['--json_file', self.tmp_json_file]
 
         return cmd
 
@@ -176,10 +169,10 @@ def {0}(**kwargs):
         # print useful links
         test_src_file = inspect.getfile(flow_test_config.__class__)
         if hasattr(flow_test_config, 'logs_link'):
-            print("\tLogs:", self.document_viewer(flow_test_config.logs_link), file=sys.stderr)
-        print("\tTest source:", self.document_viewer(test_src_file), file=sys.stderr)
-        print("\tStdout:", self.document_viewer(flow_test_config.std_out), file=sys.stderr)
-        print("\tStderr:", self.document_viewer(flow_test_config.std_err), file=sys.stderr)
+            print("\tLogs:", self.document_viewer(flow_test_config.logs_link))
+        print("\tTest source:", self.document_viewer(test_src_file))
+        print("\tStdout:", self.document_viewer(flow_test_config.std_out))
+        print("\tStderr:", self.document_viewer(flow_test_config.std_err))
 
         elapsed_time = None
         verification_time = None
@@ -189,8 +182,9 @@ def {0}(**kwargs):
             with open(flow_test_config.std_out, 'w') as std_out_f, open(flow_test_config.std_err, 'w') as std_err_f:
                 start_time = time.monotonic()
                 process = subprocess.Popen(cmd, stdout=std_out_f, stderr=std_err_f,
-                                           universal_newlines=True, shell=False, cwd=self.execution_directory)
-                _, _ = process.communicate(timeout=getattr(flow_test_config, "timeout", None))
+                                           universal_newlines=True, shell=False, cwd=self.execution_directory,
+                                           env=os.environ | flow_test_config.get_extra_run_env())
+                _, _ = process.communicate(timeout=getattr(flow_test_config, "timeout", 30 * 60))
                 elapsed_time = time.monotonic() - start_time
 
             verification_start_time = time.monotonic()
@@ -199,8 +193,12 @@ def {0}(**kwargs):
             if expected_return is not None:
                 raise Exception("assert_expected_return_code should not return. It should assert if needed")
 
-            if self.tmp_json_file and os.path.isfile(self.tmp_json_file):
-                flow_test_config.run_data = load_completion_report(self.tmp_json_file)
+            try:
+                flow_test_config.run_data = FireXRunData.load_from_logs_dir(
+                    flow_test_config.logs_link
+                )
+            except (OSError, AttributeError): # flow_test_config is badly implemented, so need AttributeError
+                flow_test_config.run_data = None
 
             if self.is_submit_command(flow_test_config) and process.returncode == 0 and \
                     self.is_instance_of_intercept(flow_test_config) and \
@@ -231,15 +229,14 @@ def {0}(**kwargs):
             print('\tException: {}: {}'.format(type(e).__name__, e), file=sys.stderr)
             raise
         finally:
-            self.on_test_exit(flow_test_config.std_out, flow_test_config.std_err)
 
             try:
-                flow_test_config.cleanup()
-            except Exception as cleanup_e:
-                print(f'Exception during flow test cleanup: {cleanup_e}')
-
-            if self.tmp_json_file and os.path.exists(self.tmp_json_file):
-                os.unlink(self.tmp_json_file)
+                self.on_test_exit(flow_test_config.std_out, flow_test_config.std_err)
+            finally:
+                try:
+                    flow_test_config.cleanup()
+                except Exception as cleanup_e:
+                    print(f'Exception during flow test cleanup: {cleanup_e}')
 
             # report on time
             if elapsed_time is not None:
@@ -248,7 +245,7 @@ def {0}(**kwargs):
                     overhead = " +(%.1fs)" % verification_time
                     if overhead != " +(0.0s)":
                         msg += overhead
-                print(msg, file=sys.stderr)
+                print(msg)
 
     def cleanup_after_timeout(self, std_out, std_err):
         pass
@@ -262,13 +259,12 @@ def {0}(**kwargs):
                 std_out_content = std_out_f.read()
                 firex_id = get_firex_id_from_output(std_out_content)
                 if firex_id:
-                    print("\tFireX ID: " + firex_id, file=sys.stderr)
+                    print("\tFireX ID: " + firex_id)
                     if self.is_public:
                         install_configs = load_new_install_configs(firex_id,
                                                                    get_log_dir_from_output(std_out_content),
                                                                    get_cloud_ci_install_config_path())
-                        print(f'\tLogs URL: {install_configs.get_logs_root_url()}', file=sys.stderr)
-                        print(f"\tFlame: {install_configs.run_url}", file=sys.stderr)
+                        print(f'\tLogs URL: {install_configs.get_logs_root_url()}')
+                        print(f"\tFlame: {install_configs.run_url}")
         except Exception as e:
-            print(e)
-            pass
+            print(e)
\ No newline at end of file
